// calcule la solution approchée de l'équation de la chaleur 
// conditions au bord de Dirichlet homogènes
// donnée initiale u0
// terme source g
clear;
clf;
exec g.sci;
exec u0.sci;
exec Fourier.sci;


// Parametres
K=64; // nombre de modes de Fourier
m=200; // 1/m pas de la discrétisation spatiale
T=0.5; //taille de l'intervalle de temps
p=100; // T/p pas de la discrétisation en temps
x=0:1/m:1; // une périodes 
t=0:T/p:T;


// Calcul de la somme partielle de Fourier de u
y0=u0(x);
n=-K/2:1:K/2;
s=zeros(p+1,m+1);
// cas où g=0
//for n=-K/2:1:K/2;
 //   cn=Fourier(u0,n,K);
 //   s=s+cn*exp(-4*%pi^2*n.^2*t')*exp(2*%pi*%i*n*x);
//end

// cas où g non nulle
a=zeros(p+1,1);
G=zeros(K+1,p+1);
for n=-K/2:1:K/2;
    cn=Fourier(u0,n,K); // coeff de Fourier de u0
    yn=exp(-4*%pi^2*n^2*t'); // coeff multiplicateur
    G(n+K/2+1,:)=exp(-2*%pi*%i*n*(0:K-1)/K)*g(t,(0:K-1)/K)'/K; 
    // G matrice des coeffs de Fourier en x de g, pour différents temps,
    // on décale de K/2+1 les fréquences pour avoir des indices positifs 
    for j=1:p+1
          a(j)=j*T/p*sum(flipdim(G(n+K/2+1,1:j),2)*yn(1:j));
    end
    // a(j) = 2ème terme dans le coefficient de Fourier de u
    s=s+(cn*yn+a)*exp(2*%pi*%i*n*x);

end

// Visualisation des modes de Fourier de g 
N=-K/2:K/2;
figure(0);
plot(N/K,abs(G(N+K/2+1,1)),N/K,abs(G(N+K/2+1,p+1)))
xlabel('Frequences');
ylabel('Modes de Fourier')
title('Spectre de la fonction g')
legend('t=0','t=T')

// Graphique : surface
figure(1)
surf(x,t,s);
title('La solution de l''equation de la chaleur', 'fontsize',3)
xlabel('x')
ylabel('t')
zlabel('u(t,x)')

// Graphique : courbes superposées
figure(2);
plot(x,s(1,:),x,s(10,:),x,s(20,:),x,s(30,:))
legend('t=0','t=0,1','t=0.2','t=0.3')
xlabel('x')
ylabel('u(t,x)')
title('Evolution de la solution au cours du temps')


